/************************************************************************
 *   IRC - Internet Relay Chat, support/verify.c
 *   This file is copyright (C) 2001 Andrew Suffield
 *                                    <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include "config.h"
#include "verify.h"
#include "irc_string.h"

#include <assert.h>
#include <ctype.h>

/* 
 * valid_hostname - check hostname for validity
 *
 * Inputs       - pointer to user
 * Output       - YES if valid, NO if not
 * Side effects - NONE
 *
 * NOTE: this doesn't allow a hostname to begin with a dot and
 * will not allow more dots than chars.
 */
int
valid_hostname(const char *hostname)
{
  int dots = 0;
  int chars = 0;
  const char *p = hostname;

  assert(0 != p);

  if ('.' == *p)
    return 0;

  while (*p)
    {
      if (!IsHostChar(*p))
	return 0;
      if ('.' == *p++)
	++dots;
      else
	++chars;
    }
  return (0 == dots || chars < dots) ? 0 : 1;
}

/* 
 * valid_username - check username for validity
 *
 * Inputs       - pointer to user
 * Output       - YES if valid, NO if not
 * Side effects - NONE
 * 
 * Absolutely always reject any '*' '!' '?' '@' '.' in an user name
 * reject any odd control characters names.
 */
int
valid_username(const char *username)
{
  const char *p = username;
  int uppercases = 0;
  assert(0 != p);

  if ('~' == *p)
    ++p;
  /* 
   * reject usernames that don't start with an alphanum
   * i.e. reject jokers who have '-@somehost' or '.@somehost'
   * or "-hi-@somehost", "h-----@somehost" would still be accepted.
   *
   * -Dianora
   */
  if (!IsAlNum(*p))
    return 0;

  while (*++p)
    {
      if (!IsUserChar(*p))
	return 0;
      if (isupper(*p))
	uppercases++;
    }
  if (uppercases > 2)
    return 0;
  return 1;
}
